classdef instr_HMO < handle
    % INSTR_HMO Provides essential functions for setting up and controling the instrument
    
       properties
            acqTime = 20e-6; % Acquisition time in seconds
            sampleRate = 5e9;  % Sample rate in Hertz
            instrID = '';   % VISA instrument ID (eg. 'rto-400101')
            connected = 0;  % Intsrument connected (0/1)
       end
       properties (Hidden)
            connection; 
       end
       methods
       % methods, including the constructor are defined in this block
           %Constructor
           function obj = instr_HMO(VxiID)
                obj.instrID = VxiID;
                disp('Loading HMO driver...');
                obj.connection = icdevice('hmo.mdd', VxiID);
           end
           
           function obj = Connect(obj)
           %   CONNECT Establish the connection to the device
           try
                disp('Connecting to HMO...');
                connect(obj.connection);
                state = obj.connection.Status;
                if strcmp(state,'open')
                    obj.connected = 1;
                else 
                    obj.connected = 0;
                end
           catch err
                msgbox(err.message);
                obj.connected = 0;
           end
           end
           
           function Disconnect(obj)
               %   DISCONNECT Disconnect from the instrument                        
                disconnect(obj.connection);
                state = obj.connection.Status;
                if strcmp(state,'open')
                    obj.connected = 1;                    
                else 
                    obj.connected = 0;                    
                end   
                delete(obj.connection);
           end           
           
           function ConfigureChannels(obj)
                % CONFIGURECHANNELS Configure all channels defined in the property channelSets
                try
                    numChannels = size(obj.channelSets);
                    for channel = 1:numChannels(2)
                        invoke(obj.connection, 'ConfigureChannel', channel,obj.channelSets(channel).enabled,obj.channelSets(channel).vertRange,obj.channelSets(channel).vertOffset,obj.channelSets(channel).vertCoupling); %
                    end
                catch
                   obj.getErrorMsg();
                end    
           end
           
           function [wav, actPoi] = GetAcquisition(obj, channelNo, numPoints)    
               %GETACQUISITION Do one measurement and return waveform data
               %    W = GETACQUISITION(channelNo) returns the waveform data
               %    for the specified channel(s) eg. 1,2,3,4 or [1 2], [1 3 4], ...
               try
                   invoke(obj.connection,'WriteInstrData','Run');
                   pause(1);
                   invoke(obj.connection,'InitiateAcquisition',0);     
                   pause(1)
                   channel = channelNo(1);                   
                   tmp(length(channelNo),:) = zeros(1,int64(numPoints));   % preinitialize
                   pause(1);
                   [~,actPoi,~,~] = invoke(obj.connection, 'ReadWaveform', channel,numPoints,5000,tmp(1,:));
                   for i=1:length(channelNo)
                    channel = channelNo(i);
                    [tmp(i,:),actPoi,~,~] = invoke(obj.connection, 'FetchWaveform',  channel,actPoi,tmp(i,:));
                    wav(i,1:actPoi) = tmp(i,1:actPoi);
                   end   
                   invoke(obj.connection,'WriteInstrData','Run');                  
               catch
                   obj.getErrorMsg();
               end
           end
           
           function ConfigureForMiracast(obj, trigLvl,~)
               %    CONFIGUREFORMIRACAST COnfigures the instrument for the
               %    Miracast measurement
               %    trigLvl         Level to which the trigger will be set
                disp('Configuring HMO...');
               % Trigger A            
                obj.writeRead('TRIG:A:TYPE EDGE;*OPC?');
                obj.writeRead('TRIG:A:SOUR CH1;*OPC?');                 
                obj.writeRead('TRIG:A:MODE NORM;*OPC?');                
                obj.writeRead('TRIG:A:EDGE:SLOP POS;*OPC?');                
                obj.writeRead('TRIG:A:EDGE:COUP DC;*OPC?');                
                obj.writeRead('TRIG:A:EDGE:FILT:LPAS ON;*OPC?');                
                obj.writeRead(['TRIG:A:LEV1 ' num2str(str2double(trigLvl)) ';*OPC?']);                
                obj.writeRead('ACQ:WRAT MSAM;*OPC?');                      
                obj.writeRead('TIM:POS 0.0;*OPC?');                
                obj.writeRead('TIM:SCAL 0.1;*OPC?');                
                obj.writeRead('TIM:REF 50;*OPC?');
                % Channel 2                
                % Enable, vertRange: 0.4V,vertOffset: -2 V,verCoupling: DCL
                invoke(obj.connection, 'ConfigureChannel',  2,1,0.4,-2,3);
                obj.writeRead('CHAN2:SCAL 0.05;*OPC?');                
                obj.writeRead('CHAN2:TYPE SAMP;*OPC?');
                obj.writeRead('CHAN2:BAND B20;*OPC?');
                % Channel 1
                % Enable, vertRange: 0.4V,vertOffset: -2 V,verCoupling: DCL
                invoke(obj.connection, 'ConfigureChannel',  1,1,0.4,-2,3);
                obj.writeRead('CHAN1:SCAL 0.05;*OPC?');                
                obj.writeRead('CHAN1:TYPE SAMP;*OPC?');
                obj.writeRead('CHAN1:BAND B20;*OPC?');                
                obj.writeRead(['TRIG:A:LEV1 ' num2str(str2double(trigLvl)) ';*OPC?']);    
           end
           
           function getErrorMsg(obj)
               % GETERRORMSG Query the instrument error state
                invoke(obj.connection,'WriteInstrData','Syst:err?');
                read = 0;
                Buf = '';
                ret = invoke(obj.connection,'ReadInstrData',256,Buf,read);
                disp(['Instrument error status:' ret]);
           end
           
           function reset(obj)
               % RESET resets the device
                obj.writeRead('*rst; *OPC?');
                pause(1);
           end
           
           function ret = writeRead(obj,msg)
               % WRITEREAD writes a scpi command to the device and queries the
               % answer
                invoke(obj.connection,'WriteInstrData',msg);
                read = 0;
                Buf = '';
                ret = invoke(obj.connection,'ReadInstrData',256,Buf,read);
           end
       end       
end 