function plotData(allData, pos, delay, sampleTime,plotNr)
% PlOTDATA Plots all the data, measurements and enables to save the data
% allData       Cell that contains vectors for each measurement 
%                   (phone, rf and tv delay). For 2 measurements it is
%                   a cell that contains 2 items consisting of 3 vectors
% pos           Position as an index in the data vector of the
%                   first occurence of the data (for phone, rf, tv)
% delay         Vector containing the phone, rf and tv display
%                   delay for miracast in seconds
% sampleTime    sampling time between each point
% plotNr        Optional - Specifies that not last signal data in allData
%               will be plotted. If e.g. 2: allData{2} will be plotted
f = figure;

if exist('plotNr','var')
    data = allData{plotNr};
else
    data = allData{length(allData)};
end
set(gcf,'units','normalized','outerposition',[0.12 0.12 0.8 0.5]);    
ax_Graph = axes('Position',[.1 .2 .85 .65]);
t = 0:sampleTime:(1.2-sampleTime);
plot(t,data(1,:),'b');
hold on;
plot(t,data(2,:),'y');
plot(t,data(3,:),'r');
set(ax_Graph,'XLim',[0 1.2])
% Plot vertical lines for the delay calculation
yLim = get(ax_Graph,'YTick');
yL = yLim(1); % Y-Limit Low
yH = yLim(length(yLim))*1.2; % Y-Limit High
set(ax_Graph,'YLim',[yL yH]);
len = floor(length(data(1,:))); % Data Length 

% Add Vertical Lines with description for the three points in time
line([pos(1) pos(1)],[yL yH],'LineStyle','--','color','black');
txt1 = ['tsrc = ' num2str(round(pos(1)*1e3)) ' ms  '];
text(pos(1) - floor(len/100)*sampleTime,yH,txt1,'Rotation',90,'FontWeight','bold','HorizontalAlignment','right');

line([pos(2) pos(2)],[yL yH],'LineStyle','--','color','black');
txt2 = ['trf = ' num2str(round(pos(2)*1e3)) ' ms  '];
text(pos(2)+ floor(len/100)*sampleTime,yH,txt2,'Rotation',90,'FontWeight','bold','HorizontalAlignment','right');

line([pos(3) pos(3)],[yL yH*1.2],'LineStyle','--','color','black');
txt3 = ['tsnk = ' num2str(round(pos(3)*1e3)) ' ms  '];
text(pos(3) - floor(len/100)*sampleTime,yH,txt3,'Rotation',90,'FontWeight','bold','HorizontalAlignment','right');

% Add Labels
xlabel(gca,'Time / s','FontWeight','bold','FontSize',12);
ylabel(gca,'Voltage / V','FontWeight','bold','FontSize',12);

% Add textboxes with info about the delta times

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0,0.021,0.09,0.05],...
                'Style','text',...   
                'horizontalAlignment','right',...
                'BackgroundColor',[0.8 0.8 0.8],...
                'String','Source->RF ',...
                'Enable','on');

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.1,0.03,0.1,0.05],...
                'Style','edit',...                
                'Enable','on',...
                'String',num2str(mean(delay(:,1))));

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.205,0.021,0.03,0.05],...
                'Style','text',...   
                'horizontalAlignment','left',...
                'BackgroundColor',[0.8 0.8 0.8],...
                'String','ms',...
                'Enable','on');          
            
uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.25,0.021,0.09,0.05],...
                'Style','text',...   
                'horizontalAlignment','right',...
                'BackgroundColor',[0.8 0.8 0.8],...
                'String','Source->HDMI ',...
                'Enable','on');

tmp = mean(delay(:,3));            
if tmp ~=0
    str = num2str(tmp,'%.4f');
else
    str = 'n.a.';
end
uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.35,0.03,0.1,0.05],...
                'Style','edit',...                
                'Enable','on',...
                'String',str);

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.455,0.021,0.03,0.05],...
                'Style','text',...   
                'horizontalAlignment','left',...
                'BackgroundColor',[0.8 0.8 0.8],...
                'String','ms',...
                'Enable','on');    

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.5,0.021,0.09,0.05],...
                'Style','text',...   
                'horizontalAlignment','right',...
                'BackgroundColor',[0.8 0.8 0.8],...
                'String','Source->Sink ',...
                'Enable','on');          
            
uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.6,0.03,0.1,0.05],...
                'Style','edit',...                
                'Enable','on',...
                'String',num2str(mean(delay(:,2))));

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.705,0.021,0.03,0.05],...
                'Style','text',...   
                'horizontalAlignment','left',...
                'BackgroundColor',[0.8 0.8 0.8],...
                'String','ms',...
                'Enable','on');    

uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.75,0.03,0.09,0.05],...
                'Style','pushbutton',... 
                'String','Save data',...
                'Enable','on',...
                'Callback',{@save,allData,delay});
            
uicontrol('Parent',f,...
                'Units','normalized',...
                'Position',[0.85,0.03,0.09,0.05],...
                'Style','pushbutton',... 
                'String','Cancel',...
                'Enable','on',...
                'Callback',@close_fig);

end

function close_fig(~,~)
    % CLOSE Closes the figure
    close(gcf);
end

function save(~,~,allData,delay)
% SAVE save all the data provided
% allData       Contains all the data to be saved
% Delay         Contains all the information about the measured delay
    filename = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    for ii = 1:length(allData)
        data = allData{ii};
        fileID = fopen([filename '_' num2str(ii) 'src.bin'],'w');
        fwrite(fileID,data(1,:),'double');
        fclose(fileID);   
        fileID = fopen([filename '_' num2str(ii) '_rf.bin'],'w');
        fwrite(fileID,data(2,:),'double');
        fclose(fileID);   
        fileID = fopen([filename '_' num2str(ii) 'snk_.bin'],'w');
        fwrite(fileID,data(3,:),'double');
        fclose(fileID);   
    end
    fileID = fopen([filename '_Results.txt'],'w');
    str{1} = ['Source2RF Delay:  ' num2str(transpose(delay(:,1)),' %07.3f')];
    str{2} = ['Source2RF Mean:   ' num2str(mean(delay(:,1)),' %07.3f')];
    str{3} = ['Source2HD Delay:  ' num2str(transpose(delay(:,3)),' %07.3f')];
    str{4} = ['Source2HD Mean:   ' num2str(mean(delay(:,3)),' %07.3f')];
    str{5} = ['Source2Snk Delay: ' num2str(transpose(delay(:,2)),' %07.3f')];
    str{6} = ['Source2Snk Mean:  ' num2str(mean(delay(:,2)),' %07.3f')];
    fprintf(fileID,'%s\r\n',str{:,:});
    fclose(fileID); 
    
    disp('************ Data saved ************');
end